package com.ctgu.pro_eld_mqtt_compare.service;

import com.ctgu.pro_eld_mqtt_compare.api.ApiService;
import com.ctgu.pro_eld_mqtt_compare.request.GetInfoVO;
import com.ctgu.pro_eld_mqtt_compare.request.PublishMQTTVO;
import com.ctgu.pro_eld_mqtt_compare.utils.TAG;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @ClassName CompareService
 * @Author Alex2
 * @Date 2025/7/4 10:34
 **/
@Slf4j
@Service
public class CompareService {

    private static ApiService apiService;

    private final ObjectMapper mapper = new ObjectMapper();

    private static final Map<String, List<String>> KEY_FIELDS = new HashMap<>();

    static {
        KEY_FIELDS.put(TAG.VEHICLE, TAG.VEHICLE_FIELDS);
        KEY_FIELDS.put(TAG.DEVICE, TAG.DEVICE_FIELDS);
        KEY_FIELDS.put(TAG.BUSINESS, TAG.BUSINESS_FIELDS);
        KEY_FIELDS.put(TAG.DRIVER, TAG.DRIVER_FIELDS);
        KEY_FIELDS.put(TAG.VEHICLE_DEVICE, TAG.VEHICLE_DEVICE_FIELDS);
        KEY_FIELDS.put(TAG.DRIVER_VEHICLE, TAG.DRIVER_VEHICLE_FIELDS);
        KEY_FIELDS.put(TAG.BUSINESS_DRIVER, TAG.BUSINESS_DRIVER_FIELDS);
    }

    @Autowired
    public CompareService(ApiService apiService) {
        CompareService.apiService = apiService;
    }

    // 生成 value 组合
    private static String getValue(Map<String, Object> item, List<String> fieldNames) {
        return fieldNames.stream()
                .map(field -> String.valueOf(item.get(field)))
                .collect(Collectors.joining("-"));
    }

    // 获取 value 字段
    private static List<String> getValueField(String tableName) {
        List<String> valueFields = KEY_FIELDS.get(tableName);
        if (valueFields == null) {
            throw new IllegalArgumentException("未配置字段列表: " + tableName);
        }
        return valueFields;
    }

    // 比较
    public PublishMQTTVO compare(Integer customerId, String tableName) throws JsonProcessingException {
        // Pakistan API
        GetInfoVO driverVehicle = new GetInfoVO(customerId, tableName);
        List<Object> postResultList = apiService.getInfoPakistan(driverVehicle).block();
        List<Map<String, Object>> pakistanList = mapper.convertValue(postResultList, new TypeReference<List<Map<String, Object>>>() {
        });

        // Pro API
        List<Object> getResultList = apiService.getInfoPro(customerId, tableName).block();
        String json2 = mapper.writeValueAsString(getResultList);
        List<Map<String, Object>> proList = mapper.readValue(json2, new TypeReference<List<Map<String, Object>>>() {
        });
        return compareList(customerId, tableName, pakistanList, proList);
    }

    // 比较列表
//    public void compareList(Integer customerId, String tableName, List<Map<String, Object>> pakistanList, List<Map<String, Object>> proList) {
//        List<String> valueFields = getValueField(tableName);
//        // 构建 key 集合
//        Set<String> pakistanSet = pakistanList.stream().map(item -> getValue(item, valueFields)).collect(Collectors.toSet());
//        log.info("customerId = {} - tableName = {} - pakistanSet = {}", customerId, tableName, pakistanSet);
//        Set<String> proSet = proList.stream().map(item -> getValue(item, valueFields)).collect(Collectors.toSet());
//        log.info("customerId = {} - tableName = {} - proSet = {}", customerId, tableName, proSet);
//
//        // 输出所有不同项，key 不同时存在于 pakistanSet 和 proSet 中
//        List<Map<String, Object>> diffList = Stream.concat(pakistanList.stream(), proList.stream())
//                .filter(item -> {
//                    String key = getValue(item, valueFields);
//                    return !(pakistanSet.contains(key) && proSet.contains(key));
//                })
//                .collect(Collectors.toList());
//        log.info("customerId = {} - tableName = {} - differList = {}", customerId, tableName, diffList);
//
//        if ("vehicle".equals(tableName) || "device".equals(tableName) || "business".equals(tableName) || "driver".equals(tableName)) {
//            Set<Object> differIds = diffList.stream().map(item -> item.get("id")).collect(Collectors.toSet());
//            log.info("customerId = {} - tableName = {} - differIds = {}", customerId, tableName, differIds);
//        }else if ("vehicle_device".equals(tableName) || "driver_vehicle".equals(tableName) || "business_driver".equals(tableName)) {
//
//        }
//    }

    public PublishMQTTVO compareList(Integer customerId, String tableName, List<Map<String, Object>> pakistanList, List<Map<String, Object>> proList) {
        List<String> valueFields = getValueField(tableName);

        Map<String, Map<String, Object>> pakistanMap = pakistanList.stream()
                .collect(Collectors.toMap(item -> getValue(item, valueFields), item -> item,(existing, replacement) -> existing));
        log.info("customerId = {} - tableName = {} - pakistanSet = {}", customerId, tableName, pakistanMap.keySet());

        Map<String, Map<String, Object>> proMap = proList.stream()
                .collect(Collectors.toMap(item -> getValue(item, valueFields), item -> item,(existing, replacement) -> existing));
        log.info("customerId = {} - tableName = {} - proSet = {}", customerId, tableName, proMap.keySet());

        // 合并所有 key
        Set<String> allKeys = new HashSet<>();
        allKeys.addAll(pakistanMap.keySet());
        allKeys.addAll(proMap.keySet());

        // 仅保留出现在一侧的 key（即差异 key）
        List<Map<String, Object>> diffList = allKeys.stream()
                .filter(key -> !(pakistanMap.containsKey(key) && proMap.containsKey(key)))
                .map(key -> pakistanMap.getOrDefault(key, proMap.get(key))) // 取出差异项
                .collect(Collectors.toList());

        log.info("customerId = {} - tableName = {} - differList = {}", customerId, tableName, diffList);

        Set<Integer> differIds = diffList.stream()
                .map(item -> String.valueOf(item.get("id")))
                .map(Integer::valueOf)
                .collect(Collectors.toSet());
        log.info("customerId = {} - tableName = {} - differIds = {}", customerId, tableName, differIds);
        return new PublishMQTTVO(customerId, tableName, differIds);
    }
}